/* 
** compat.c: 
** copyright: (c) 2012 by Hemmingsen Kim
** email: kmgh10@hotmail.com
** This program is free software; you can redistribute it and/or 
** modify  it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
*/
/*compat.c holds redefinition of gtk2 syntax to gtk1*/
#include "compat.h"
#include "pixmaps.h"

/// Create a new hbox with an image and a label packed into it
/// and return the box.
GtkWidget *xpm_label_box( GtkWidget *parent, char *xpm_filename, gchar *label_text, int height, int width ) {
	GtkWidget *box1;
	GtkWidget *label;
	GtkWidget *pixmapwid;
	GdkPixmap *pixmap;
	GdkBitmap *mask = NULL;
	GdkColormap *colormap;
	GdkPixbuf *pixbuf;
	FILE *file = NULL;
	/// for buildin images
	const char **xpm = NULL;
	box1 = gtk_hbox_new (FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (box1), 2);
	colormap = gtk_widget_get_colormap (parent);
	
	file = fopen(xpm_filename, "rb");
	
	if (!file) {
		if (!label_text) {
			xpm = error_xpm;
		}
		/// find default build in image if external fail
		else if ( !strcmp(label_text, "Yes") ) 
			xpm = yes_xpm;
		else if (!strcmp(label_text, "OK") )
			xpm = ok_xpm;
		else if (!strcmp(label_text, "Cancel") || !strcmp(label_text, "No"))
			xpm = no_xpm;
		else if (!strcmp(label_text, "Help")) 
			xpm = help_xpm;		
		else
			xpm = error_xpm;
			
		pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
		pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
				SMALL_WIDTH, SMALL_HEIGHT, GDK_INTERP_TILES);
		gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
		pixmapwid = gtk_pixmap_new (pixmap, mask);
		gdk_pixbuf_unref(pixbuf);
			
	} else {
		
		fclose(file);	
#ifdef USE_GTK2	
		pixbuf = gdk_pixbuf_new_from_file(xpm_filename, NULL);
#else
		pixbuf = gdk_pixbuf_new_from_file(xpm_filename);
#endif	

		if (( ! height == 0) && ( ! width == 0)) {
			pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
				width, height, GDK_INTERP_TILES);
		} else {
			pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
				SMALL_WIDTH, SMALL_HEIGHT, GDK_INTERP_TILES);
		}
		gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
		pixmapwid = gtk_pixmap_new (pixmap, mask);
		gdk_pixbuf_unref(pixbuf);
	}
		
	gtk_box_pack_start(GTK_BOX (box1), pixmapwid, FALSE, FALSE, 0);
	gtk_widget_show(pixmapwid);
	if ( ! (pixmap == NULL)) {	/// precaution static builds might segfault
		gdk_pixmap_unref(pixmap);
	}
	if ( ! (mask == NULL)) {	/// precaution static builds might segfault
		gdk_pixmap_unref(mask); 
	}
	/// skip label if not needed
	if (label_text) {
		label = gtk_label_new (label_text);
		gtk_box_pack_start(GTK_BOX (box1), label, FALSE, FALSE, 3);
		gtk_widget_show(label);
	}
	
	return(box1);
}

///gtk1+2 common compat
GtkWidget *gtk_image_new_from_file_wrap(char *str, int height, int width) {
	
	GdkPixbuf *pixbuf;
	GtkWidget *pixmapwid;
	GdkPixmap *pixmap = NULL;
	GdkBitmap *mask = NULL;
	FILE *file = NULL;
	const char **xpm = NULL; /// for buildin images
	
	///we need to check if file found
	file = fopen(str, "rb");
	
	if (!file) {
		xpm = error_xpm;
				
		pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
		pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
				SMALL_WIDTH, SMALL_HEIGHT, GDK_INTERP_TILES);		
	} else {
		fclose(file);	
#ifndef USE_GTK2
		pixbuf = gdk_pixbuf_new_from_file(str);
#else
		pixbuf = gdk_pixbuf_new_from_file(str, NULL);
#endif
		if (( height <= 0) && ( width <= 0 ))  {
			pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
				SMALL_WIDTH, SMALL_HEIGHT, GDK_INTERP_TILES);
		} else {
			pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
				width, height, GDK_INTERP_TILES);
		}
	}
	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
	pixmapwid = gtk_pixmap_new (pixmap, mask);
	gdk_pixbuf_unref(pixbuf);
	
	gtk_widget_show(pixmapwid);
	if ( ! (pixmap == NULL)) {	/// precaution static builds might segfault
		gdk_pixmap_unref(pixmap);
	}
	if ( ! (mask == NULL)) {	/// precaution static builds might segfault
		gdk_pixmap_unref(mask); 
	}
		
	return (pixmapwid);
}


#ifndef USE_GTK2
#define _GNU_SOURCE
#include <stdio.h>
#include "attributes.h"
#include <sys/stat.h>	//needed if we use "stat"
#include <ctype.h>	//needed for 'toupper'


/// gtk1 wrapper
GtkWidget *gtk_text_view_new(void) {
	GtkWidget	*Widget = NULL;
	Widget=gtk_text_new(NULL,NULL);
	gtk_text_set_editable ( GTK_TEXT (Widget), TRUE);
	gtk_text_set_word_wrap( GTK_TEXT (Widget), TRUE);
	gtk_text_set_point( GTK_TEXT (Widget), 0);
		    	
	return(Widget);
}

/// gtk1 wrapper
GtkWidget *gtk_button_new_from_stock (char *stock_id) {
	
	GtkWidget *Widget = NULL;
	GtkWidget *box1 = NULL;
	GtkWidget *icon = NULL; 
	GtkWidget *label = NULL;
	const char **xpm = NULL;
	GdkPixmap *pixmap = NULL;
	GdkBitmap *mask = NULL;
	GdkPixbuf *pixbuf;	

	if ( !strcmp(stock_id, GTK_STOCK_YES) ) {
		label = gtk_label_new ("Yes");
		if (find_pixmap("gtk-yes.png")) {
			icon = gtk_image_new_from_file_wrap(find_pixmap("gtk-yes.png"),
				SMALL_WIDTH, SMALL_HEIGHT);
		} else {
			xpm = yes_xpm;
		}
	} else if ( !strcmp(stock_id, GTK_STOCK_OK) ) {
		label = gtk_label_new ("OK");
		if (find_pixmap("gtk-ok.png")) {
			icon = gtk_image_new_from_file_wrap(find_pixmap("gtk-ok.png"),
				SMALL_WIDTH, SMALL_HEIGHT);
		} else {
			xpm = ok_xpm;		
		}		
	} else if ( !strcmp(stock_id, GTK_STOCK_CANCEL) ) {
		label = gtk_label_new ("Cancel");
		if (find_pixmap("gtk-cancel.png")) {
			icon = gtk_image_new_from_file_wrap(find_pixmap("gtk-cancel.png"),
				SMALL_WIDTH, SMALL_HEIGHT);
		} else {
			xpm = cancel_xpm;		
		}	
	} else if ( !strcmp(stock_id, GTK_STOCK_NO) ) {
		label = gtk_label_new ("No");
		if (find_pixmap("gtk-no.png")) {
			icon = gtk_image_new_from_file_wrap(find_pixmap("gtk-no.png"),
				SMALL_WIDTH, SMALL_HEIGHT);
		} else {
			xpm = no_xpm;		
		}	
	} else if ( !strcmp(stock_id, GTK_STOCK_HELP) )  {
		label = gtk_label_new ("Help");
		if (find_pixmap("gtk-help.png")) {
			icon = gtk_image_new_from_file_wrap(find_pixmap("gtk-help.png"),
				SMALL_WIDTH, SMALL_HEIGHT);	
		} else {
			xpm = help_xpm;		
		}	
	} else {
			//guess we never end here!
			label = gtk_label_new ("Unlabeled");
	}			
	
	///use build in image
	if (xpm) {
		pixbuf = gdk_pixbuf_new_from_xpm_data (xpm);
		pixbuf = gdk_pixbuf_scale_simple(pixbuf, 
			SMALL_WIDTH, SMALL_HEIGHT, GDK_INTERP_TILES);
		gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 128);
		icon = gtk_pixmap_new (pixmap, mask);
		gdk_pixbuf_unref(pixbuf);
	}
	
	Widget = gtk_button_new();
	box1 = gtk_hbox_new (FALSE, 0);
	gtk_container_add(GTK_CONTAINER(Widget), box1);
	gtk_box_pack_start(GTK_BOX(box1), icon, FALSE, FALSE, 3);
	gtk_box_pack_start(GTK_BOX(box1), label, FALSE, FALSE, 3);
			
	return(Widget);
}

/// my helpers...
char *replace_str(char *str, char *orig, char *rep) {
  static char buffer[4096];
  char *p;

  if(!(p = strstr(str, orig)))  // Is 'orig' even in 'str'?
    return str;

  strncpy(buffer, str, p-str); // Copy characters from 'str' start to 'orig' st$
  buffer[p-str] = '\0';

  sprintf(buffer+(p-str), "%s%s", rep, p+strlen(orig));

  return buffer;
}

/// gtk1 wrapper
GtkWidget *gtk_image_menu_item_new_from_stock (char *stock_id, int *dummy) {
	
	GtkWidget	*menu_item = NULL;
	GtkWidget	*hbox;
	GtkWidget	*label;	
	GtkWidget	*img;
	char		*label_old = stock_id;
	
	if (strncmp("gtk-", stock_id, 4) == 0){
		label_old = replace_str(label_old, "gtk-", "");
		label_old[0] = toupper(label_old[0]); /* capitalize */
	}	
			
	hbox = gtk_hbox_new(FALSE, 0);	
	label = gtk_label_new(label_old);
	img = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON, GTK_ICON_SIZE_BUTTON);
	gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0); 
  	gtk_widget_show(img);
  	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
	gtk_widget_show(label);
	
	menu_item = gtk_menu_item_new();
	gtk_container_add(GTK_CONTAINER(menu_item), hbox);
	gtk_widget_show(hbox);
			
	return menu_item;
}

///gtk1 wrapper
GtkWidget *gtk_image_new_from_stock (char *stock_id, int height, int width) {
	
	GtkWidget *Icon = NULL;
	char *filename = NULL;
	///the stock=" syntax
	if (strncmp("gtk-", stock_id, 4) == 0){
		filename=stock_id;
		asprintf(&filename, "%s%s", filename, ".png");
	} else {
	///the icon=" syntax - hopefully - as we dont check!
		asprintf(&stock_id, "%s%s", stock_id, ".png");
		filename = stock_id;
	}
	
	Icon = gtk_image_new_from_file_wrap(find_pixmap(filename), height, width);
	return (Icon);
}

void action_chooserselection_made( GtkWidget *w, GtkFileSelection *fs ) {
	char *selected_path;
	selected_path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	///2015: gtk2 filechooser strip off trailing "/" so need to do that for gtk1 too
	char* pos;
	pos=selected_path+(strlen(selected_path))-1;
	if (strncasecmp(pos, "/", 1) == 0) {
		selected_path[strlen(selected_path)-1] = 0;
	}
	///
	CHOOSERCONT = selected_path;
	return;
}

void action_dirselected (GtkWidget *widget, gint row, gint column, GdkEventButton *bevent, gpointer user_data) {
	char *selected_dir,*temp = NULL;
	GtkFileSelection *fs = NULL;
	fs = GTK_FILE_SELECTION (user_data);
	gtk_clist_get_text (GTK_CLIST (fs->dir_list), row, 0, &temp);
  	selected_dir = g_strdup (temp);
  	CHOOSERCONT = selected_dir;
	return;
}

void action_fileselected (GtkWidget *widget, gint row, gint column, GdkEventButton *bevent, gpointer user_data) {
	char *selected_file,*temp = NULL;
	GtkFileSelection *fs = NULL;
	fs = GTK_FILE_SELECTION (user_data);
	gtk_clist_get_text (GTK_CLIST (fs->file_list), row, 0, &temp);
  	selected_file = g_strdup (temp);
  	CHOOSERCONT = selected_file;
	return;
}

#endif


